#
# Copyright (c) 2000, 2099, trustbe and/or its affiliates. All rights reserved.
# TRUSTBE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
#
#

from mesh.sidecar.sidecar import open_sidecar

__all__ = (
    "open_sidecar"
)


def init():
    """ init function """
    pass
